@@ -1,13 +1,14 @@
-# Copyright (c) 2018 Ultimaker B.V.
+# Copyright (c) 2020 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
-from json import JSONDecodeError
-from typing import Any, Dict, List, Optional, Set, Tuple, cast, TYPE_CHECKING
+
 import json
 import os
 import shutil
-import zipfile
 import tempfile
 import urllib.parse  # For interpreting escape characters using unquote_plus.
+import zipfile
+from json import JSONDecodeError
+from typing import Any, Dict, List, Optional, Set, Tuple, cast, TYPE_CHECKING
 
 from PyQt5.QtCore import pyqtSlot, QObject, pyqtSignal, QUrl, pyqtProperty
 
@@ -95,8 +96,10 @@ class PackageManager(QObject):
     def packagesWithUpdate(self) -> Set[str]:
         return self._packages_with_update_available
 
-    ## Alternative way of setting the available package updates without having to check all packages in the cloud.
     def setPackagesWithUpdate(self, packages: Set[str]):
+        """Alternative way of setting the available package updates without having to check all packages in the
+        cloud. """
+
         self._packages_with_update_available = packages
         self.packagesWithUpdateChanged.emit()
 
@@ -156,9 +159,13 @@ class PackageManager(QObject):
         # Load the bundled packages:
         self._bundled_package_dict = {}
         for search_path in self._bundled_package_management_file_paths:
-            with open(search_path, "r", encoding = "utf-8") as f:
-                self._bundled_package_dict.update(json.load(f, encoding = "utf-8"))
-                Logger.log("i", "Loaded bundled packages data from %s", search_path)
+            try:
+                with open(search_path, "r", encoding = "utf-8") as f:
+                    self._bundled_package_dict.update(json.load(f))
+                    Logger.log("i", "Loaded bundled packages data from %s", search_path)
+            except UnicodeDecodeError:
+                Logger.logException("e", "Can't decode package management files. File is corrupt.")
+                return
 
         # Need to use the file lock here to prevent concurrent I/O from other processes/threads
         container_registry = self._application.getContainerRegistry()
@@ -167,7 +174,7 @@ class PackageManager(QObject):
                 # Load the user packages:
                 with open(cast(str, self._user_package_management_file_path), "r", encoding="utf-8") as f:
                     try:
-                        management_dict = json.load(f, encoding="utf-8")
+                        management_dict = json.load(f)
                     except JSONDecodeError:
                         # The file got corrupted, ignore it. This happens extremely infrequently.
                         # The file will get overridden once a user downloads something.
@@ -188,10 +195,10 @@ class PackageManager(QObject):
             if bundled_package_dict is None:
                 continue
 
-            result = self._comparePackageVersions(installed_package_dict["package_info"],
+            should_install = self._shouldInstallCandidate(installed_package_dict["package_info"],
                                                   bundled_package_dict["package_info"])
             # The bundled package is newer
-            if result <= 0:
+            if not should_install:
                 self._to_remove_package_set.add(package_id)
                 continue
 
@@ -203,49 +210,52 @@ class PackageManager(QObject):
             if bundled_package_dict is None:
                 continue
 
-            result = self._comparePackageVersions(to_install_package_dict["package_info"],
+            should_install = self._shouldInstallCandidate(to_install_package_dict["package_info"],
                                                   bundled_package_dict["package_info"])
             # The bundled package is newer
-            if result <= 0:
+            if not should_install:
+                Logger.info(
+                    "Ignoring package {} since it's sdk or package version is lower than the bundled package",
+                    package_id
+                )
                 to_remove_package_ids.add(package_id)
                 continue
         for package_id in to_remove_package_ids:
             del self._to_install_package_dict[package_id]
 
     # Compares the SDK versions and the package versions of the two given package info dicts.
-    # Returns -1, 0, 1 indicating if the versions in dict1 is lower than, equal to, or higher than dict2.
+    # Returns True if the candidate is preferred over the base
     #  - The package with the higher SDK version is considered having the higher version number. If they are the same,
-    #  - if the bundled package version is greater than or equal to the given package, -1 is returned. Otherwise, 1.
-    def _comparePackageVersions(self, info_dict1: Dict[str, Any], info_dict2: Dict[str, Any]) -> int:
-        # If the bundled version has a higher SDK version, use the bundled version by removing the installed one.
-        sdk_version1 = UMVersion(info_dict1["sdk_version"])
-        sdk_version2 = UMVersion(info_dict2["sdk_version"])
-        if sdk_version1 < sdk_version2:
-            return -1
+    #  - if the based package version is greater than or equal to the given package, -1 is returned. Otherwise, 1.
+    def _shouldInstallCandidate(self, candidate_dict: Dict[str, Any], base_dict: Dict[str, Any]) -> bool:
+        # If the base version has a higher SDK version, use the based version by removing the candidate one.
+        sdk_version_candidate = UMVersion(candidate_dict["sdk_version"])
+        if not self.isPackageCompatible(sdk_version_candidate):
+            return False
 
-        # Remove the package with the old version to favour the newer bundled version.
-        version1 = UMVersion(info_dict1["package_version"])
-        version2 = UMVersion(info_dict2["package_version"])
-        if version1 < version2:
-            return -1
+        # Remove the package with the old version to favour the newer based version.
+        version_candidate = UMVersion(candidate_dict["package_version"])
+        version_base = UMVersion(base_dict["package_version"])
+        if version_candidate <= version_base:
+            return False
 
-        if version1 == version2:
-            return 0
-
-        return 1
+        return True
 
     def _saveManagementData(self) -> None:
         # Need to use the file lock here to prevent concurrent I/O from other processes/threads
         container_registry = self._application.getContainerRegistry()
         with container_registry.lockFile():
-            with open(cast(str,self._user_package_management_file_path), "w", encoding = "utf-8") as f:
-                data_dict = {"version": PackageManager.Version,
-                             "installed": self._installed_package_dict,
-                             "to_remove": list(self._to_remove_package_set),
-                             "to_install": self._to_install_package_dict,
-                             "dismissed": list(self._dismissed_packages)}
-                json.dump(data_dict, f, sort_keys = True, indent = 4)
-                Logger.log("i", "Package management file %s was saved", self._user_package_management_file_path)
+            try:
+                with open(cast(str,self._user_package_management_file_path), "w", encoding = "utf-8") as f:
+                    data_dict = {"version": PackageManager.Version,
+                                 "installed": self._installed_package_dict,
+                                 "to_remove": list(self._to_remove_package_set),
+                                 "to_install": self._to_install_package_dict,
+                                 "dismissed": list(self._dismissed_packages)}
+                    json.dump(data_dict, f, sort_keys = True, indent = 4)
+                    Logger.log("i", "Package management file %s was saved", self._user_package_management_file_path)
+            except EnvironmentError as e:  # Can't save for whatever reason (permissions, missing directory, hard drive full, etc).
+                Logger.error("Unable to save package management file to {path}: {err}".format(path = self._user_package_management_file_path, err = str(e)))
 
     # (for initialize) Removes all packages that have been scheduled to be removed.
     def _removeAllScheduledPackages(self) -> None:
@@ -320,13 +330,12 @@ class PackageManager(QObject):
 
         return all_installed_ids
 
-    # Get a list of packages that the user has installed.
-    def getUserInstalledPackages(self) -> List[str]:
-        return [package for package in self._installed_package_dict]
-
-    ## Get a list of tuples that contain the package ID and version.
-    #  Used by the Marketplace to check which packages have updates available.
     def getAllInstalledPackageIdsAndVersions(self) -> List[Tuple[str, str]]:
+        """Get a list of tuples that contain the package ID and version.
+
+        Used by the Marketplace to check which packages have updates available.
+        """
+
         package_ids_and_versions = []  # type: List[Tuple[str, str]]
         all_installed_ids = self.getAllInstalledPackageIDs()
         for package_id in all_installed_ids:
@@ -399,10 +408,19 @@ class PackageManager(QObject):
     def isPackageInstalled(self, package_id: str) -> bool:
         return self.getInstalledPackageInfo(package_id) is not None
 
-    ## Schedules the given package file to be installed upon the next start.
-    # \return The to-be-installed package_id or None if something went wrong
+    @pyqtSlot(QUrl)
+    def installPackageViaDragAndDrop(self, file_url: str) -> None:
+        """This is called by drag-and-dropping curapackage files."""
+        filename = QUrl(file_url).toLocalFile()
+        return self.installPackage(filename)
+
     @pyqtSlot(str)
     def installPackage(self, filename: str) -> Optional[str]:
+        """Schedules the given package file to be installed upon the next start.
+
+        :return: The to-be-installed package_id or None if something went wrong
+        """
+
         has_changes = False
         package_id = ""
         try:
@@ -485,8 +503,9 @@ class PackageManager(QObject):
             self._packages_with_update_available.add(package_id)
             self.packagesWithUpdateChanged.emit()
 
-    ##  Is the package an user installed package?
     def isUserInstalledPackage(self, package_id: str) -> bool:
+        """Is the package an user installed package?"""
+
         return package_id in self._installed_package_dict
 
     # Removes everything associated with the given package ID.
@@ -593,24 +612,32 @@ class PackageManager(QObject):
         license_string = None
         def is_license(zipinfo: zipfile.ZipInfo) -> bool:
             return os.path.basename(zipinfo.filename).startswith("LICENSE")
-        with zipfile.ZipFile(filename) as archive:
-            # Go through all the files and use the first successful read as the result
-            license_files = sorted(filter(is_license, archive.infolist()), key = lambda x: len(x.filename))  # Find the one with the shortest path.
-            for file_info in license_files:
-                Logger.log("d", "Found potential license file '{filename}'".format(filename = file_info.filename))
-                try:
-                    with archive.open(file_info.filename, "r") as f:
-                        data = f.read()
-                    license_string = data.decode("utf-8")
-                    break
-                except:
-                    Logger.logException("e", "Failed to load potential license file '%s' as text file.", file_info.filename)
-                    license_string = None
+        try:
+            with zipfile.ZipFile(filename) as archive:
+                # Go through all the files and use the first successful read as the result
+                license_files = sorted(filter(is_license, archive.infolist()), key = lambda x: len(x.filename))  # Find the one with the shortest path.
+                for file_info in license_files:
+                    Logger.log("d", "Found potential license file '{filename}'".format(filename = file_info.filename))
+                    try:
+                        with archive.open(file_info.filename, "r") as f:
+                            data = f.read()
+                        license_string = data.decode("utf-8")
+                        break
+                    except:
+                        Logger.logException("e", "Failed to load potential license file '%s' as text file.", file_info.filename)
+                        license_string = None
+        except zipfile.BadZipFile as e:
+            Logger.error("Package is corrupt: {err}".format(err = str(e)))
+            license_string = None
+        except UnicodeDecodeError:
+            Logger.error("Package filenames are not UTF-8 encoded! Encoding unknown.")
+            license_string = None
         return license_string
 
-    ##  Find the package files by package_id by looking at the installed folder
     @staticmethod
     def getPackageFiles(package_id) -> List[Tuple[str, List[str]]]:
+        """Find the package files by package_id by looking at the installed folder"""
+
         data_storage_dir = os.path.abspath(Resources.getDataStoragePath())
 
         os_walk = []
@@ -630,9 +657,10 @@ class PackageManager(QObject):
 
         return result
 
-    ##  Return container ids for contents found with package_id
     @staticmethod
     def getPackageContainerIds(package_id: str) -> List[str]:
+        """Return container ids for contents found with package_id"""
+
         package_files = PackageManager.getPackageFiles(package_id)
         ids = []
         for root_path, file_names in package_files:
@@ -643,9 +671,10 @@ class PackageManager(QObject):
                     ids.append(id)
         return ids
 
-    ##  Try to return Id for given path by looking at its existence in the mimetype database
     @staticmethod
     def convertPathToId(path: str) -> str:
+        """Try to return Id for given path by looking at its existence in the mimetype database"""
+
         mime = None
         try:
             mime = MimeTypeDatabase.getMimeTypeForFile(path)
